<?php

namespace Drupal\zuora\Rest;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Http\ClientFactory;
use Drupal\zuora\Exception\ZuoraException;
use Drupal\zuora\ZuoraClientBase;

class Client extends ZuoraClientBase {

  /**
   *
   */
  const ZUORA_PRODUCTION = 'https://api.zuora.com/rest';

  /**
   *
   */
  const ZUORA_SANDBOX = 'https://apisandbox-api.zuora.com/rest';

  /**
   *
   */
  const ZUORA_API_VERSION = 'v1';

  protected $httpClient;

  public function __construct(ConfigFactoryInterface $config_factory, ClientFactory $client_factory) {
    parent::__construct($config_factory);

    $base_uri = ($this->isSandboxed()) ? self::ZUORA_SANDBOX : self::ZUORA_PRODUCTION;
    $this->httpClient = $client_factory->fromOptions([
      'base_uri' => $base_uri,
    ]);
  }


  /**
   * Perform GET request.
   *
   * @param $uri
   * @param array|NULL $body
   *
   * @return array|mixed
   */
  public function get($uri) {
    return $this->httpRequest('GET', $uri);
  }

  /**
   * Perform POST request.
   *
   * @param $uri
   * @param array $body
   *
   * @return array|mixed
   */
  public function post($uri, array $body) {
    return $this->httpRequest('POST', $uri, $body);
  }

  /**
   * Perform PUT request.
   *
   * @param $uri
   * @param array $body
   *
   * @return array|mixed
   */
  public function put($uri, array $body) {
    return $this->httpRequest('PUT', $uri, $body);
  }

  /**
   * Builds a drupal_http_request call.
   *
   * @param $type
   * @param $uri
   * @param array|NULL $body
   *
   * @throws ZuoraException
   *
   * @return array|mixed
   */
  public function httpRequest($type, $uri, array $body = null) {
    $response = $this->httpClient->request($type, '/rest/' . self::ZUORA_API_VERSION . $uri, [
      'json' => $body,
      'headers' => $this->httpHeaders(),
    ]);
    $data = json_decode($response->getBody()->getContents(), TRUE);

    if ($response->getStatusCode() == 200) {
      return $data;
    }
    else {
      throw new ZuoraException("Zuora API error: {$data['error']}");
    }

  }

  /**
   * Returns HTTP request headers.
   *
   * @return array
   */
  protected function httpHeaders() {
    return array(
      'Accept' => 'application/json',
      'Content-Type' => 'application/json',
      'apiAccessKeyId' => $this->zuoraConfig->get('access_key_id'),
      'apiSecretAccessKey' => $this->zuoraConfig->get('access_secret_key'),
    );
  }

}
